@@ -33,7 +33,7 @@ from datetime import date, datetime
 import enum
 import operator
 from pathlib import Path
-from typing import Any, Dict, List, Optional, Set, Tuple, Type, Union
+from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Type, Union
 from uuid import UUID
 
 
@@ -43,6 +43,7 @@ except ImportError:
     from typing_compat import get_origin
 
 from pydantic import EmailStr
+from pydantic.fields import SHAPE_LIST
 from sqlalchemy import (
     Boolean,
     Column,
@@ -61,6 +62,7 @@ from sqlalchemy import (
 )
 from sqlalchemy.sql import select
 
+from pygasus.field.helpers import update_pygasus_field
 from pygasus.model import Field, Model, Sequence
 from pygasus.storage.abc import AbstractStorageEngine
 from pygasus.storage.sql.query_builder import SQLQueryBuilder
@@ -93,15 +95,15 @@ class SQLStorageEngine(AbstractStorageEngine):
         self.memory = False
         self.tables = {}
         self.uniques = {}
+        self.futures = set()
 
     def init(
         self,
         file_name: Union[str, Path, None] = None,
         memory: bool = False,
-        logging: bool = True,
+        logging: Union[bool, Callable[[str, Tuple[Any]], None]] = True,
     ):
-        """
-        Initialize the storage engine.
+        """Initialize the storage engine.
 
         Args:
             file_name (str or Path): the file_name in which the database
@@ -142,8 +144,10 @@ class SQLStorageEngine(AbstractStorageEngine):
         # Intercept requests to log them, if set.
         @event.listens_for(self.engine, "before_cursor_execute")
         def log_query(conn, cr, statement, parameters, *_):
+            log = self.logging
+            log = log if callable(log) else print
             if self.logging:
-                print(statement.strip(), parameters)
+                log(statement.strip(), parameters)
 
         self.connection = self.engine.connect()
         self.metadata = MetaData()
@@ -186,8 +190,9 @@ class SQLStorageEngine(AbstractStorageEngine):
         # Browse model fields, creating columsn, indexes and constraints.
         indexes = []
         for name, field in model.__fields__.items():
-            model.__pygasus__[name].__model__ = model
-            model.__pygasus__[name].__storage__ = self
+            model.__pygasus__[name].bind(model, self)
+            update_pygasus_field(model.__pygasus__[name])
+            pygasus = model.__pygasus__[name]
             o_type = field.outer_type_
             f_type = field.type_
             info = field.field_info
@@ -210,6 +215,28 @@ class SQLStorageEngine(AbstractStorageEngine):
                 )
                 continue
             elif isinstance(f_type, type) and issubclass(f_type, Model):
+                # If the field is optional (and back isn't) do not add.
+                back = self.get_back_field(model, field, f_type)
+
+                # Choose the owner field.
+                extra = info.extra
+                back_extra = back.field_info.extra
+                if extra.get("owner") and not back_extra.get("owner"):
+                    extra["owner"] = True
+                    back_extra["owner"] = False
+                elif not extra.get("owner") and back_extra.get("owner"):
+                    extra["owner"] = False
+                    back_extra["owner"] = True
+                elif pygasus.required and not back.required:
+                    extra["owner"] = True
+                    back_extra["owner"] = False
+                elif not pygasus.required and back.required:
+                    extra["owner"] = False
+                    back_extra["owner"] = True
+
+                if not extra.get("owner"):
+                    continue
+
                 # Create foreign key fields.
                 to_name = getattr(
                     f_type.__config__, "model_name", f_type.__name__.lower()
@@ -226,17 +253,19 @@ class SQLStorageEngine(AbstractStorageEngine):
                             f"{name}_{pname}",
                             None,
                             ForeignKey(f"{to_name}.{pname}"),
+                            nullable=not field.required,
                         )
                     )
 
                 # Add sort options.
-                columns.append(
-                    Column(
-                        f"{name}__index",
-                        Integer,
-                        nullable=False,
+                if back.shape == SHAPE_LIST:
+                    columns.append(
+                        Column(
+                            f"{name}__index",
+                            Integer,
+                            nullable=False,
+                        )
                     )
-                )
                 continue
 
             # Handle enumerations here.
@@ -346,6 +375,7 @@ class SQLStorageEngine(AbstractStorageEngine):
             info = field.field_info
             o_type = field.outer_type_
             f_type = field.type_
+            pygasus = getattr(model, field.name)
             if o_type is Sequence[f_type]:
                 exclude.add(name)
                 linked.add(name)
@@ -357,7 +387,11 @@ class SQLStorageEngine(AbstractStorageEngine):
                 sql[name] = custom.to_storage(value)
 
             if issubclass(f_type, Model):
-                right = attrs[name]
+                right = attrs.get(name, field.get_default())
+                if not isinstance(right, Model):  # Skip here (non valid).
+                    exclude.add(name)
+                    continue
+
                 primary_keys = {
                     name: field
                     for name, field in f_type.__fields__.items()
@@ -390,6 +424,13 @@ class SQLStorageEngine(AbstractStorageEngine):
 
         # Validate the model object.
         obj = model(**attrs)
+        for name, attr in attrs.items():
+            pygasus = getattr(model, name)
+            pygasus.validate_update(obj, None, attr)
+
+        for name, attr in attrs.items():
+            pygasus = getattr(model, name)
+            pygasus.perform_update(obj, None, attr)
 
         # Create a row.
         table = self.tables[model_name]
@@ -561,7 +602,7 @@ class SQLStorageEngine(AbstractStorageEngine):
         columns, tables = self._get_columns_for(model)
         query = select(*columns)
         for join in tables:
-            query = query.join(table)
+            query = query.join(join)
         where = []
         for column, value in kwargs.items():
             method = getattr(
@@ -635,15 +676,23 @@ class SQLStorageEngine(AbstractStorageEngine):
         ]
         return objs
 
-    def update(self, model, instance, key, old_value, new_value):
+    def update(
+        self,
+        model: Type["Model"],
+        instance: "Model",
+        key: str,
+        old_value: Any,
+        new_value: Any,
+    ):
         """Update an instance attribute.
 
         Args:
             model (subclass of Model): the model class.
-            instance (Model: the model object.
+            instance (Model): the model object.
             key (str): the name of the attribute to modify.
             old_value (Any): the value before the modification.
             new_value (Any): the value after the modification.
+            check_update (bool): if True (default), check the update.
 
         """
         model_name = getattr(
@@ -689,8 +738,36 @@ class SQLStorageEngine(AbstractStorageEngine):
         if custom:
             new_value = custom.to_storage(new_value)
 
-        # Send the query.
         sql_columns = {key: new_value}
+        f_type = field.type_
+        pygasus = model.__pygasus__[field.name]
+        if issubclass(f_type, Model):
+            if not field.field_info.extra.get("owner", True) and new_value:
+                old_value = getattr(new_value, pygasus.__back__.name, None)
+                return self.update(
+                    type(new_value),
+                    new_value,
+                    pygasus.__back__.name,
+                    old_value,
+                    instance,
+                )
+
+            sql_columns = {}
+            linked_keys = {
+                linked_name: linked_field
+                for linked_name, linked_field in f_type.__fields__.items()
+                if linked_field.field_info.extra.get("primary_key")
+            }
+
+            for pname, pfield in linked_keys.items():
+                value = new_value and getattr(new_value, pname) or None
+                sql_columns[f"{field.name}_{pname}"] = value
+
+        # Check that this update can be performed.
+        pygasus.validate_update(instance, old_value, new_value)
+        pygasus.perform_update(instance, old_value, new_value)
+
+        # Send the query.
         update = (
             sql_table.update().where(*sql_primary_keys).values(**sql_columns)
         )
@@ -715,6 +792,7 @@ class SQLStorageEngine(AbstractStorageEngine):
         for name, field in model.__fields__.items():
             info = field.field_info
             pk = info.extra.get("primary_key", False)
+            pygasus = model.__pygasus__[name]
             if pk:
                 value = getattr(instance, name)
                 # Convert other field types.
@@ -726,6 +804,8 @@ class SQLStorageEngine(AbstractStorageEngine):
                 sql_primary_keys.append(getattr(sql_table.c, name) == value)
                 pks.append(value)
 
+            pygasus.validate_delete(instance)
+
         # Remove this object from cache.
         self.cache.pop((model,) + tuple(pks), None)
 
@@ -733,6 +813,10 @@ class SQLStorageEngine(AbstractStorageEngine):
         delete = sql_table.delete().where(*sql_primary_keys)
         self.connection.execute(delete)
 
+        # Apply other updates if necessary.
+        for pygasus in model.__pygasus__.values():
+            pygasus.perform_delete(instance)
+
     def bulk_delete(self, model: Type[Model], query) -> int:
         """Select one or more models with the specified query and delete them.
 
@@ -776,7 +860,9 @@ class SQLStorageEngine(AbstractStorageEngine):
         ret = self.connection.execute(sql)
         return ret.rowcount
 
-    def _get_columns_for(self, model: Type[Model]) -> Tuple[Column]:
+    def _get_columns_for(
+        self, model: Type[Model], recursion=True
+    ) -> Tuple[Column]:
         """Return the relevant columns for the specified model.
 
         Args:
@@ -797,19 +883,22 @@ class SQLStorageEngine(AbstractStorageEngine):
             f_type = field.type_
             if issubclass(f_type, Model) and o_type is not Sequence[f_type]:
                 # Join up.
-                tables.append(
-                    self.tables[
-                        getattr(
-                            f_type.__config__,
-                            "model_name",
-                            f_type.__name__.lower(),
-                        )
-                    ]
-                )
+                if recursion:
+                    tables.append(
+                        self.tables[
+                            getattr(
+                                f_type.__config__,
+                                "model_name",
+                                f_type.__name__.lower(),
+                            )
+                        ]
+                    )
 
-                other_columns, other_tables = self._get_columns_for(f_type)
-                columns.extend(other_columns)
-                tables.extend(other_tables)
+                    other_columns, other_tables = self._get_columns_for(
+                        f_type, recursion=False
+                    )
+                    columns.extend(other_columns)
+                    tables.extend(other_tables)
             elif o_type is not Sequence[f_type]:
                 columns.append(
                     getattr(table.c, name).label(f"{model_name}_{name}")
@@ -881,14 +970,33 @@ class SQLStorageEngine(AbstractStorageEngine):
                     and issubclass(f_type, Model)
                     and o_type is not Sequence[f_type]
                 ):
-                    if f_type in done:
-                        continue
+                    if not field.required:
+                        attrs[name] = field.get_default()
 
-                    obj = self._build_objects_from_row(
-                        row, (f_type,), done=tuple(done)
-                    )
-                    others.append(obj)
-                    attrs[name] = obj
+                        # Prepare the future.
+                        pkeys = tuple(
+                            row[f"{model_name}_{pk.name}"]
+                            for pk in model.__fields__.values()
+                            if pk.field_info.extra.get("primary_key", False)
+                        )
+                        link_pkeys = tuple(
+                            row[f"{name}_{pk.name}"]
+                            for pk in f_type.__fields__.values()
+                            if pk.field_info.extra.get("primary_key", False)
+                        )
+                        self.futures.add(
+                            (model, pkeys, name, f_type, link_pkeys)
+                        )
+                    else:
+                        obj = self._build_objects_from_row(
+                            row, (f_type,), done=tuple(done)
+                        )
+                        others.append(obj)
+                        attrs[name] = obj
+
+                        # Check whether this update would be possible.
+                        pygasus = model.__pygasus__[name]
+                        pygasus.validate_update(model, None, obj)
                 elif issubclass(f_type, enum.Enum):
                     try:
                         value = f_type(value)
@@ -932,8 +1040,14 @@ class SQLStorageEngine(AbstractStorageEngine):
                         col.parent = obj
                         obj._exists = True
 
+                # Update Pygasus fields.
+                for pygasus in type(obj).__pygasus__.values():
+                    value = attrs.get(pygasus.name)
+                    pygasus.perform_update(obj, None, value)
+
             objs.extend(others)
 
+        self._apply_futures()
         if not objs:
             objs = [None]
 
@@ -949,3 +1063,14 @@ class SQLStorageEngine(AbstractStorageEngine):
     def uuid_to_storage(self, model: Model, field: Field, value: UUID) -> str:
         """Convert a UUID to a str representation."""
         return value.hex
+
+    def _apply_futures(self):
+        """Try to apply the futures."""
+        for model, pks, key, link_model, link_pks in tuple(self.futures):
+            obj = self.cache.get((model,) + pks)
+            to_link = self.cache.get((link_model,) + link_pks)
+            if obj is not None and to_link is not None:
+                obj._exists = False
+                setattr(obj, key, to_link)
+                obj._exists = True
+                self.futures.discard((model, pks, key, link_model, link_pks))
